<?php

namespace Drupal\computed_field;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\computed_field\Annotation\ComputedField;
use Drupal\computed_field\Plugin\ComputedField\ComputedFieldPluginInterface;
use Drupal\Core\Entity\EntityTypeInterface;

/**
 * Manages discovery and instantiation of Computed Field plugins.
 */
class ComputedFieldManager extends DefaultPluginManager {

  /**
   * Constructs a new ComputedFieldManagerManager.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   The cache backend.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(
    \Traversable $namespaces,
    CacheBackendInterface $cache_backend,
    ModuleHandlerInterface $module_handler
  ) {
    parent::__construct(
      'Plugin/ComputedField',
      $namespaces,
      $module_handler,
      ComputedFieldPluginInterface::class,
      ComputedField::class
    );

    $this->alterInfo('computed_field_info');
    $this->setCacheBackend($cache_backend, 'computed_field_plugins');
  }

  /**
   * Gets the definitions of attachable plugins.
   *
   * @return array
   *   An array in the same format as static::getDefinitions(), containing only
   *   the definitions of plugins which are attachable, that is, which do not
   *   declare entity types to attach computed fields to and are instead
   *   configured with config entities.
   */
  public function getUiDefinitions(): array {
    return array_filter($this->getDefinitions(), function ($definition) {
      return empty($definition['no_ui']);
    });
  }

  /**
   * Gets the definitions of base field automatic plugins.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type to get plugins for.
   *
   * @return array
   *   An array of plugin definitions in the same format as
   *   static::getDefinitions().
   */
  public function getAutomaticBasePluginDefinitions(EntityTypeInterface $entity_type): array {
    $automatic_base_computed_field_plugins = [];
    foreach ($this->getDefinitions() as $plugin_id => $definition) {
      if (!isset($definition['attach'])) {
        continue;
      }

      if ($definition['attach']['scope'] != 'base') {
        continue;
      }

      if (!isset($definition['attach']['entity_types'][$entity_type->id()])) {
        continue;
      }

      $automatic_base_computed_field_plugins[$plugin_id] = $definition;
    }

    return $automatic_base_computed_field_plugins;
  }

  /**
   * Gets the definitions of bundle field automatic plugins.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type to get plugins for.
   * @param string $bundle
   *   The bundle name.
   *
   * @return array
   *   An array of plugin definitions in the same format as
   *   static::getDefinitions().
   */
  public function getAutomaticBundlePluginDefinitions(EntityTypeInterface $entity_type, string $bundle): array {
    $automatic_bundle_computed_field_plugins = [];
    foreach ($this->getDefinitions() as $plugin_id => $definition) {
      if (!isset($definition['attach'])) {
        continue;
      }

      if ($definition['attach']['scope'] != 'bundle') {
        continue;
      }

      if (!isset($definition['attach']['entity_types'][$entity_type->id()])) {
        continue;
      }
      if (!in_array($bundle, $definition['attach']['entity_types'][$entity_type->id()])) {
        continue;
      }

      $automatic_bundle_computed_field_plugins[$plugin_id] = $definition;
    }

    return $automatic_bundle_computed_field_plugins;
  }

  /**
   * {@inheritdoc}
   */
  public function processDefinition(&$definition, $plugin_id) {
    parent::processDefinition($definition, $plugin_id);

    // Automatically attaching plugins must specify all the keys in the 'attach'
    // array.
    if (isset($definition['attach'])) {
      foreach (['scope', 'field_name', 'entity_types'] as $key) {
        if (!isset($definition['attach'][$key])) {
          throw new InvalidPluginDefinitionException($plugin_id, "Key '$key' must be specified in the 'attach' array in plugin $plugin_id.");
        }
      }

      // Automatic bundle fields must specify bundles.
      if ($definition['attach']['scope'] == 'bundle') {
        foreach ($definition['attach']['entity_types'] as $entity_type_id => $bundles) {
          if (empty($bundles)) {
            throw new InvalidPluginDefinitionException("The entity_types property for '$entity_type_id' must have a list of bundles if scope is 'bundle' in plugin definition of '$plugin_id'.");
          }
        }
      }
    }
  }

}
